@@ -1,5 +1,5 @@
 /*
-  Copyright(C) 2018 Kouhei Sutou <kou@clear-code.com>
+  Copyright(C) 2018-2022  Sutou Kouhei <kou@clear-code.com>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Lesser General Public
@@ -150,7 +150,7 @@ grn_romaji_hepburn_consonant(grn_ctx *ctx,
         /* U+304B HIRAGANA LETTER KA ..
          * U+3054 HIRAGANA LETTER GO */
         const char *gk = "gk";
-        return gk[current[2] % 2];
+        return (unsigned char)(gk[current[2] % 2]);
       } else if (0x95 <= current[2] && current[2] <= 0x9e) {
         /* U+3055 HIRAGANA LETTER SA ..
          * U+305E HIRAGANA LETTER ZO */
@@ -162,13 +162,13 @@ grn_romaji_hepburn_consonant(grn_ctx *ctx,
           return 'j';
         } else {
           const char *zs = "zs";
-          return zs[current[2] % 2];
+          return (unsigned char)(zs[current[2] % 2]);
         }
       } else if (0x9f <= current[2] && current[2] <= 0xa9) {
         /* U+305F HIRAGANA LETTER TA ..
          * U+3069 HIRAGANA LETTER DO */
         const char *tdtjxtztdtd = "tdtjxtztdtd";
-        return tdtjxtztdtd[current[2] - 0x9f];
+        return (unsigned char)(tdtjxtztdtd[current[2] - 0x9f]);
       } else if (0xaa <= current[2] && current[2] <= 0xae) {
         /* U+306A HIRAGANA LETTER NA ..
          * U+306E HIRAGANA LETTER NO */
@@ -177,7 +177,7 @@ grn_romaji_hepburn_consonant(grn_ctx *ctx,
         /* U+306F HIRAGANA LETTER HA ..
          * U+307D HIRAGANA LETTER PO */
         const char *phb = "phb";
-        return phb[current[2] % 3];
+        return (unsigned char)(phb[current[2] % 3]);
       } else if (0xbe <= current[2] && current[2] <= 0xbf) {
         /* U+307E HIRAGANA LETTER MA ..
          * U+307F HIRAGANA LETTER MI */
@@ -212,7 +212,8 @@ grn_romaji_hepburn_consonant(grn_ctx *ctx,
       } else if (current[2] == 0x93) {
         /* U+3093 HIRAGANA LETTER N */
         const unsigned char *next = current + char_length;
-        size_t next_char_length = grn_charlen_(ctx, next, end, GRN_ENC_UTF8);
+        size_t next_char_length =
+          (size_t)grn_charlen_(ctx, next, end, GRN_ENC_UTF8);
         if (grn_romaji_hepburn_is_pbm(next, next_char_length)) {
           return 'm';
         } else {
@@ -237,7 +238,7 @@ grn_romaji_hepburn_consonant(grn_ctx *ctx,
         /* U+30AB KATAKANA LETTER KA ..
          * U+30B4 KATAKANA LETTER GO */
         const char *gk = "gk";
-        return gk[current[2] % 2];
+        return (unsigned char)(gk[current[2] % 2]);
       } else if (0xb5 <= current[2] && current[2] <= 0xbe) {
         /* U+30B5 KATAKANA LETTER SA ..
          * U+30BE KATAKANA LETTER ZO */
@@ -249,7 +250,7 @@ grn_romaji_hepburn_consonant(grn_ctx *ctx,
           return 'j';
         } else {
           const char *zs = "zs";
-          return zs[current[2] % 2];
+          return (unsigned char)(zs[current[2] % 2]);
         }
       } else if (current[2] == 0xbf) {
         /* U+30BF KATAKANA LETTER TA */
@@ -261,7 +262,7 @@ grn_romaji_hepburn_consonant(grn_ctx *ctx,
         /* U+30C0 KATAKANA LETTER DA ..
          * U+30C9 KATAKANA LETTER DO */
         const char *dtjxtztdtd = "dtjxtztdtd";
-        return dtjxtztdtd[current[2] - 0x80];
+        return (unsigned char)(dtjxtztdtd[current[2] - 0x80]);
       } else if (0x8a <= current[2] && current[2] <= 0x8e) {
         /* U+30CA KATAKANA LETTER NA ..
          * U+30CE KATAKANA LETTER NO */
@@ -270,7 +271,7 @@ grn_romaji_hepburn_consonant(grn_ctx *ctx,
         /* U+30CF KATAKANA LETTER HA ..
          * U+30DD KATAKANA LETTER PO */
         const char *bph = "bph";
-        return bph[current[2] % 3];
+        return (unsigned char)(bph[current[2] % 3]);
       } else if (0x9e <= current[2] && current[2] <= 0xa2) {
         /* U+30DE KATAKANA LETTER MA ..
          * U+30E2 KATAKANA LETTER MO */
@@ -298,7 +299,8 @@ grn_romaji_hepburn_consonant(grn_ctx *ctx,
       } else if (current[2] == 0xb3) {
         /* U+30F3 KATAKANA LETTER N */
         const unsigned char *next = current + char_length;
-        size_t next_char_length = grn_charlen_(ctx, next, end, GRN_ENC_UTF8);
+        size_t next_char_length =
+          (size_t)grn_charlen_(ctx, next, end, GRN_ENC_UTF8);
         if (grn_romaji_hepburn_is_pbm(next, next_char_length)) {
           return 'm';
         } else {
@@ -341,18 +343,18 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
   const unsigned char *next = NULL;
   size_t next_char_length = 0;
   grn_bool next_small_y = GRN_FALSE;
-  char next_small_yayuyo = '\0';
+  unsigned char next_small_yayuyo = '\0';
   const char aiueo[] = "aiueo";
   const char auo[] = "auo";
   const char aaieo[] = "aaieo";
 
-  char_length = grn_charlen_(ctx, current, end, GRN_ENC_UTF8);
+  char_length = (size_t)grn_charlen_(ctx, current, end, GRN_ENC_UTF8);
   *n_used_bytes = char_length;
   *n_used_characters = 1;
 
   if (char_length == 3) {
     next = current + char_length;
-    next_char_length = grn_charlen_(ctx, next, end, GRN_ENC_UTF8);
+    next_char_length = (size_t)grn_charlen_(ctx, next, end, GRN_ENC_UTF8);
     if (next_char_length == 3) {
       switch (next[0]) {
       case 0xe3 :
@@ -363,7 +365,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           case 0x85 : /* U+3085 HIRAGANA LETTER SMALL YU */
           case 0x87 : /* U+3087 HIRAGANA LETTER SMALL YO */
             next_small_y = GRN_TRUE;
-            next_small_yayuyo = aiueo[(next[2] - 1) % 5];
+            next_small_yayuyo = (unsigned char)(aiueo[(next[2] - 1) % 5]);
             break;
           default :
             break;
@@ -374,7 +376,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           case 0xa5 : /* U+30E5 KATAKANA LETTER SMALL YU */
           case 0xa7 : /* U+30E7 KATAKANA LETTER SMALL YO */
             next_small_y = GRN_TRUE;
-            next_small_yayuyo = aiueo[(next[2] - 3) % 5];
+            next_small_yayuyo = (unsigned char)(aiueo[(next[2] - 3) % 5]);
             break;
           default :
             break;
@@ -398,12 +400,12 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
         if ((current[2] % 2) == 1) { /* SMALL */
           buffer[(*n_bytes)++] = 'x';
         }
-        buffer[(*n_bytes)++] = aiueo[(current[2] - 0x81) / 2];
+        buffer[(*n_bytes)++] = (unsigned char)(aiueo[(current[2] - 0x81) / 2]);
       } else if (0x8b <= current[2] && current[2] <= 0x94) {
         /* U+304B HIRAGANA LETTER KA ..
          * U+3054 HIRAGANA LETTER GO */
         const char *gk = "gk";
-        buffer[(*n_bytes)++] = gk[current[2] % 2];
+        buffer[(*n_bytes)++] = (unsigned char)(gk[current[2] % 2]);
         if (next_small_y &&
             (current[2] == 0x8d ||  /* U+304D HIRAGANA LETTER KI */
              current[2] == 0x8e)) { /* U+304E HIRAGANA LETTER GI */
@@ -412,7 +414,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           (*n_used_bytes) += next_char_length;
           (*n_used_characters)++;
         } else {
-          buffer[(*n_bytes)++] = aiueo[(current[2] - 0x8b) / 2];
+          buffer[(*n_bytes)++] = (unsigned char)(aiueo[(current[2] - 0x8b) / 2]);
         }
       } else if (0x95 <= current[2] && current[2] <= 0x9e) {
         /* U+3055 HIRAGANA LETTER SA ..
@@ -440,9 +442,9 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
             buffer[(*n_bytes)++] = 'j';
           } else {
             const char *zs = "zs";
-            buffer[(*n_bytes)++] = zs[current[2] % 2];
+            buffer[(*n_bytes)++] = (unsigned char)(zs[current[2] % 2]);
           }
-          buffer[(*n_bytes)++] = aiueo[(current[2] - 0x95) / 2];
+          buffer[(*n_bytes)++] = (unsigned char)(aiueo[(current[2] - 0x95) / 2]);
         }
       } else if (0x9f <= current[2] && current[2] <= 0xa9) {
         /* U+305F HIRAGANA LETTER TA ..
@@ -486,9 +488,11 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
             buffer[(*n_bytes)++] = 's';
           } else {
             const char *td____ztdtd = "td____ztdtd";
-            buffer[(*n_bytes)++] = td____ztdtd[current[2] - 0x9f];
+            buffer[(*n_bytes)++] =
+              (unsigned char)(td____ztdtd[current[2] - 0x9f]);
           }
-          buffer[(*n_bytes)++] = aaiiuuueeoo[current[2] - 0x9f];
+          buffer[(*n_bytes)++] =
+            (unsigned char)(aaiiuuueeoo[current[2] - 0x9f]);
         }
       } else if (0xaa <= current[2] && current[2] <= 0xae) {
         /* U+306A HIRAGANA LETTER NA ..
@@ -501,13 +505,13 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           (*n_used_bytes) += next_char_length;
           (*n_used_characters)++;
         } else {
-          buffer[(*n_bytes)++] = aiueo[(current[2] - 0xaa)];
+          buffer[(*n_bytes)++] = (unsigned char)(aiueo[(current[2] - 0xaa)]);
         }
       } else if (0xaf <= current[2] && current[2] <= 0xbd) {
         /* U+306F HIRAGANA LETTER HA ..
          * U+307D HIRAGANA LETTER PO */
         const char *phb = "phb";
-        buffer[(*n_bytes)++] = phb[current[2] % 3];
+        buffer[(*n_bytes)++] = (unsigned char)(phb[current[2] % 3]);
         if (next_small_y &&
             (0xb2 <= current[2] &&  /* U+3072 HIRAGANA LETTER HI .. */
              current[2] <= 0xb4)) { /* U+3074 HIRAGANA LETTER PI */
@@ -516,7 +520,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           (*n_used_bytes) += next_char_length;
           (*n_used_characters)++;
         } else {
-          buffer[(*n_bytes)++] = aiueo[(current[2] - 0xaf) / 3];
+          buffer[(*n_bytes)++] = (unsigned char)(aiueo[(current[2] - 0xaf) / 3]);
         }
       } else if (0xbe <= current[2] && current[2] <= 0xbf) {
         /* U+307E HIRAGANA LETTER MA ..
@@ -529,7 +533,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           (*n_used_bytes) += next_char_length;
           (*n_used_characters)++;
         } else {
-          buffer[(*n_bytes)++] = aiueo[(current[2] - 0xbe)];
+          buffer[(*n_bytes)++] = (unsigned char)(aiueo[(current[2] - 0xbe)]);
         }
       }
       break;
@@ -545,7 +549,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           (*n_used_bytes) += next_char_length;
           (*n_used_characters)++;
         } else {
-          buffer[(*n_bytes)++] = aiueo[(current[2] - 0x80) + 2];
+          buffer[(*n_bytes)++] = (unsigned char)(aiueo[(current[2] - 0x80) + 2]);
         }
       } else if (0x83 <= current[2] && current[2] <= 0x88) {
         /* U+3083 HIRAGANA LETTER SMALL YA ..
@@ -554,7 +558,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           buffer[(*n_bytes)++] = 'x';
         }
         buffer[(*n_bytes)++] = 'y';
-        buffer[(*n_bytes)++] = auo[(current[2] - 0x83) / 2];
+        buffer[(*n_bytes)++] = (unsigned char)(auo[(current[2] - 0x83) / 2]);
       } else if (0x89 <= current[2] && current[2] <= 0x8d) {
         /* U+3089 HIRAGANA LETTER RA ..
          * U+308D HIRAGANA LETTER RO */
@@ -566,7 +570,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           (*n_used_bytes) += next_char_length;
           (*n_used_characters)++;
         } else {
-          buffer[(*n_bytes)++] = aiueo[current[2] - 0x89];
+          buffer[(*n_bytes)++] = (unsigned char)(aiueo[current[2] - 0x89]);
         }
       } else if (0x8e <= current[2] && current[2] <= 0x92) {
         /* U+308E HIRAGANA LETTER SMALL WA ..
@@ -575,7 +579,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           buffer[(*n_bytes)++] = 'x';
         }
         buffer[(*n_bytes)++] = 'w';
-        buffer[(*n_bytes)++] = aaieo[current[2] - 0x8e];
+        buffer[(*n_bytes)++] = (unsigned char)(aaieo[current[2] - 0x8e]);
       } else if (current[2] == 0x93) {
         /* U+3093 HIRAGANA LETTER N */
         if (grn_romaji_hepburn_is_pbm(next, next_char_length)) {
@@ -606,12 +610,12 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
         if ((current[2] % 2) == 1) { /* SMALL */
           buffer[(*n_bytes)++] = 'x';
         }
-        buffer[(*n_bytes)++] = aiueo[(current[2] - 0xa1) / 2];
+        buffer[(*n_bytes)++] = (unsigned char)(aiueo[(current[2] - 0xa1) / 2]);
       } else if (0xab <= current[2] && current[2] <= 0xb4) {
         /* U+30AB KATAKANA LETTER KA ..
          * U+30B4 KATAKANA LETTER GO */
         const char *gk = "gk";
-        buffer[(*n_bytes)++] = gk[current[2] % 2];
+        buffer[(*n_bytes)++] = (unsigned char)(gk[current[2] % 2]);
         if (next_small_y &&
             (current[2] == 0xad ||  /* U+30AD KATAKANA LETTER KI */
              current[2] == 0xae)) { /* U+U+30AE KATAKANA LETTER GI */
@@ -620,7 +624,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           (*n_used_bytes) += next_char_length;
           (*n_used_characters)++;
         } else {
-          buffer[(*n_bytes)++] = aiueo[(current[2] - 0xab) / 2];
+          buffer[(*n_bytes)++] = (unsigned char)(aiueo[(current[2] - 0xab) / 2]);
         }
       } else if (0xb5 <= current[2] && current[2] <= 0xbe) {
         /* U+30B5 KATAKANA LETTER SA ..
@@ -648,9 +652,9 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
             buffer[(*n_bytes)++] = 'j';
           } else {
             const char *zs = "zs";
-            buffer[(*n_bytes)++] = zs[current[2] % 2];
+            buffer[(*n_bytes)++] = (unsigned char)(zs[current[2] % 2]);
           }
-          buffer[(*n_bytes)++] = aiueo[(current[2] - 0xb5) / 2];
+          buffer[(*n_bytes)++] = (unsigned char)(aiueo[(current[2] - 0xb5) / 2]);
         }
       } else if (current[2] == 0xbf) {
         /* U+30BF KATAKANA LETTER TA */
@@ -706,9 +710,11 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
             buffer[(*n_bytes)++] = 's';
           } else {
             const char *d____ztdtd = "d____ztdtd";
-            buffer[(*n_bytes)++] = d____ztdtd[current[2] - 0x80];
+            buffer[(*n_bytes)++] =
+              (unsigned char)(d____ztdtd[current[2] - 0x80]);
           }
-          buffer[(*n_bytes)++] = aiiuuueeoo[current[2] - 0x80];
+          buffer[(*n_bytes)++] =
+            (unsigned char)(aiiuuueeoo[current[2] - 0x80]);
         }
       } else if (0x8a <= current[2] && current[2] <= 0x8e) {
         /* U+30CA KATAKANA LETTER NA ..
@@ -721,13 +727,14 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           (*n_used_bytes) += next_char_length;
           (*n_used_characters)++;
         } else {
-          buffer[(*n_bytes)++] = aiueo[current[2] - 0x8a];
+          buffer[(*n_bytes)++] =
+            (unsigned char)(aiueo[current[2] - 0x8a]);
         }
       } else if (0x8f <= current[2] && current[2] <= 0x9d) {
         /* U+30CF KATAKANA LETTER HA ..
          * U+30DD KATAKANA LETTER PO */
         const char *bph = "bph";
-        buffer[(*n_bytes)++] = bph[current[2] % 3];
+        buffer[(*n_bytes)++] = (unsigned char)(bph[current[2] % 3]);
         if (next_small_y &&
             (0x92 <= current[2] &&  /* U+30D2 KATAKANA LETTER HI .. */
              current[2] <= 0x94)) { /* U+30D4 KATAKANA LETTER PI */
@@ -736,7 +743,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           (*n_used_bytes) += next_char_length;
           (*n_used_characters)++;
         } else {
-          buffer[(*n_bytes)++] = aiueo[(current[2] - 0x8f) / 3];
+          buffer[(*n_bytes)++] = (unsigned char)(aiueo[(current[2] - 0x8f) / 3]);
         }
       } else if (0x9e <= current[2] && current[2] <= 0xa2) {
         /* U+30DE KATAKANA LETTER MA ..
@@ -749,7 +756,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           (*n_used_bytes) += next_char_length;
           (*n_used_characters)++;
         } else {
-          buffer[(*n_bytes)++] = aiueo[current[2] - 0x9e];
+          buffer[(*n_bytes)++] = (unsigned char)(aiueo[current[2] - 0x9e]);
         }
       } else if (0xa3 <= current[2] && current[2] <= 0xa8) {
         /* U+30E3 KATAKANA LETTER SMALL YA ..
@@ -758,7 +765,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           buffer[(*n_bytes)++] = 'x';
         }
         buffer[(*n_bytes)++] = 'y';
-        buffer[(*n_bytes)++] = auo[(current[2] - 0xa3) / 2];
+        buffer[(*n_bytes)++] = (unsigned char)(auo[(current[2] - 0xa3) / 2]);
       } else if (0xa9 <= current[2] && current[2] <= 0xad) {
         /* U+30E9 KATAKANA LETTER RA ..
          * U+30ED KATAKANA LETTER RO */
@@ -770,7 +777,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           (*n_used_bytes) += next_char_length;
           (*n_used_characters)++;
         } else {
-          buffer[(*n_bytes)++] = aiueo[current[2] - 0xa9];
+          buffer[(*n_bytes)++] = (unsigned char)(aiueo[current[2] - 0xa9]);
         }
       } else if (0xae <= current[2] && current[2] <= 0xb2) {
         /* U+30EE KATAKANA LETTER SMALL WA ..
@@ -779,7 +786,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
           buffer[(*n_bytes)++] = 'x';
         }
         buffer[(*n_bytes)++] = 'w';
-        buffer[(*n_bytes)++] = aaieo[current[2] - 0xae];
+        buffer[(*n_bytes)++] = (unsigned char)(aaieo[current[2] - 0xae]);
       } else if (current[2] == 0xb3) {
         /* U+30F3 KATAKANA LETTER N */
         if (grn_romaji_hepburn_is_pbm(next, next_char_length)) {
@@ -809,7 +816,7 @@ grn_romaji_hepburn_convert(grn_ctx *ctx,
          * U+30FA KATAKANA LETTER VO */
         static char aieo[] = "aieo";
         buffer[(*n_bytes)++] = 'v';
-        buffer[(*n_bytes)++] = aieo[current[2] - 0xb7];
+        buffer[(*n_bytes)++] = (unsigned char)(aieo[current[2] - 0xb7]);
       }
       break;
     default :
